---
title: Plotting
---

## `bycon` Plot functionality

Starting with version v1.0.30 (2023-04-14) the bycon package added the ability
to produce the typical _Progenetix_-style CNV histograms and CNV sample plots. 
These plots can both be generated by adding the respective `output=...` option to
standard Beacon queries or through use of the `bycon` package to visualize data
from local storage. Plotting from local `.pgxseg` ... file is possible but so far
we don't provide a ready-made for that.

**Changes 2023-10-23** The plotting of is now hadled by dedicated endpoints in `/services`:

* `/services/collationplots` for pre-computed frequency histograms and heatstrips
* `/services/sampleplots` for query-derived sample plots (including histograms)
    - plot selection through `plotType=` with options `histoplot`, `samplesplot` and `histoheatplot`
    - defaults to `histoplot` if unspecified

### Plot types

#### CNV histograms - `/services/collationplots`

CNV histograms can be generated either (fast) for one or multiple of the "collations" _i.e._
samples sharing a common code (diagnosis, technnique...) or identifier (cell line id, 
PMID ...), or as single histogram for the output of a Beacon query.

A complete list of collations can be retrieved through the `/services/collations/`
endpoint, e.g. [/services/collations?datasetIds=progenetix&output=text](http://progenetix.org/services/collations?datasetIds=progenetix&output=text).

Please note that the `datasetIds` parameter will fall back to the default parameter
if not indicated.

##### Examples

* [/services/collationplots/?filters=NCIT:C35562,NCIT:C3709](http://progenetix.org/services/collationplots/?filters=NCIT:C35562,NCIT:C3709)
    - a combination of 2 histograms
* [/services/collationplots?filters=NCIT:C35562,NCIT:C3709&datasetIds=progenetix,cellz](http://progenetix.org/services/collationplots?filters=NCIT:C35562,NCIT:C3709&datasetIds=progenetix,cellz)
    - a combination of 2 histograms
* [/services/collationplots/?filters=pgx:icdom-85003,pgx:icdom-81703,pgx:icdom-87003,pgx:icdom-87203,pgx:icdom-94003,pgx:icdom-95003,pgx:icdom-81403&plot_title=CNV+Comparison&plot_area_height=50&plot_axis_y_max=80&plot_label_y_values=50](http://progenetix.org/services/collationplots/?filters=pgx:icdom-85003,pgx:icdom-81703,pgx:icdom-87003,pgx:icdom-87203,pgx:icdom-94003,pgx:icdom-95003,pgx:icdom-81403&plot_title=CNV+Comparison&plot_area_height=50&plot_axis_y_max=80&plot_label_y_values=50)
    - a collations based example showing the use of some extra parameters such as
        * `plot_title`
        * `plot_area_height`
        * `plot_axis_y_max` & `plot_label_y_values`

#### CNV sample plots - `/services/sampleplots`

Sample selection based plotting uses the standard bycon query stack for sample retrieval
(_i.e._ aggregation over the data model) and then generates CNV plots from the found
samples, either as clustered individual profiles or as binned frequency plot data (histograms or heatstrips).

CAVE: Sample plots are _very_ time consuming due to the retrieval and plotting of
all variants per sample.

##### Examples

* [/beacon/sampleplots?filters=pgx:icdom-95003&plot_filter_empty_samples=y&plotGeneSymbols=MYCN&plotType=samplesplot&limit=200](http://progenetix.org/beacon/sampleplots?filters=pgx:icdom-95003&plot_filter_empty_samples=y&plotGeneSymbols=MYCN&plotType=samplesplot&limit=100)
    - this example is based on the histoplot example above, but based on individual
      sample retrieval and plotting and with some plot modifications:
        * limits the output to 100 samples (`limit=100`)
        * removes samples w/o CNVs (`plot_filter_empty_samples=y`)
* [/beacon/sampleplots?filters=pgx:icdom-95003&plotGeneSymbols=MYCN&limit=100&plotType=samplesplot](http://progenetix.org/beacon/sampleplots?filters=pgx:icdom-95003&plotGeneSymbols=MYCN&limit=100&plotType=samplesplot)
    - this example gets samples for ICD-O Morphology 95003/3 (a.k.a. `pgx:icdom-95003`)
    - limits the output to the first 1000 samples (`limit=1000`)
    - adds a label for the **MYCN** gene
* [/beacon/sampleplots?filters=pgx:icdom-95003&plotGeneSymbols=MYCN&limit=100](http://progenetix.org/beacon/sampleplots?filters=pgx:icdom-95003&plotGeneSymbols=MYCN&limit=100)
    - this is the same selection and labeling but defaulting to the `histoplot`
      option since no `plotType` parameter is indicated


## Plot parameters

Plot parameters can be given both in `snake_case` and in the corresponding
`camelCase` format (`plot_area_width` or `plotAreaWidth`).


### Parameter definitions

``` yaml title="Plot parameters"
--8<-- "./bycon/config/plot_defaults.yaml"
```